library ieee;
use ieee.std_logic_1164.all;
use IEEE.numeric_std.all;
-- synthesis translate_off
LIBRARY XilinxCoreLib;
-- synthesis translate_on
library unisim;
use unisim.vcomponents.all;

entity lx9_FT245 is port(
	CLK: in std_logic;
	D: inout std_logic_vector(7 downto 0);
	RXF: in std_logic; --when high do not read
	TXE: in std_logic; --when high do not write
	RD: out std_logic;
	WR: out std_logic;
	SIWU: out std_logic;
	CLKOUT: in std_logic;
	OE: out std_logic;
	PWRSAV: out std_logic;
	LED: out std_logic_vector(1 downto 0));
end lx9_FT245;

architecture behav of lx9_FT245 is

COMPONENT fifo_generator_v9_3 IS PORT (
    wr_clk : IN STD_LOGIC;
    rd_clk : IN STD_LOGIC;
    din : IN STD_LOGIC_VECTOR(7 DOWNTO 0);
    wr_en : IN STD_LOGIC;
    rd_en : IN STD_LOGIC;
    dout : OUT STD_LOGIC_VECTOR(7 DOWNTO 0);
    full : OUT STD_LOGIC;
    empty : OUT STD_LOGIC;
    valid : OUT STD_LOGIC);
END COMPONENT;

signal blinkcounter_50: unsigned(25 downto 0):=(others => '0');
signal blinkcounter_60: unsigned(25 downto 0):=(others => '0');
signal bytecounter: unsigned(7 downto 0):=(others => '0');

signal counter_1us: integer range 0 to 99:=0;
signal send_counter: integer range 0 to 21:=21;
signal fifo_in, fifo_out: std_logic_vector(7 downto 0):=(others => '0');
signal bytes_send: std_logic_vector(7 downto 0):=(others => '0');--:=x"000102030405060708090A0B0C0D0E0F1011121314";
signal fifo_empty,fifo_full,fifo_wr,fifo_rd,TXE_buff,WR_buff,fifo_valid: std_logic:='0';

signal data_0, data_1: std_logic_vector(8 downto 0):=(others => '0');
signal data_0_8, data_1_8: std_logic_vector(7 downto 0):=(others => '0');
signal data_0_avail, data_1_avail: std_logic:='0';

begin

f0: fifo_generator_v9_3 PORT MAP(
    wr_clk => CLK,
    rd_clk => CLKOUT,
    din => fifo_in,
    wr_en => fifo_wr,
    rd_en => fifo_rd,
    dout => fifo_out,
    full => fifo_full,
    empty => fifo_empty,
    valid => fifo_valid);

RD <= '1';
SIWU <= '1';
OE <= '1';
PWRSAV <= 'Z';

LED(0) <= blinkcounter_50(25);
LED(1) <= blinkcounter_60(25);

data_0_8 <= data_0(7 downto 0);
data_1_8 <= data_1(7 downto 0);
data_0_avail <= data_0(8);
data_1_avail <= data_1(8);

process begin
	wait until rising_edge(CLK);
	blinkcounter_50 <= blinkcounter_50 +1;
	counter_1us <= counter_1us +1;
	if counter_1us = 99 then
		counter_1us <= 0;
	end if;
	fifo_wr <= '0';
	if counter_1us < 22 then
		bytecounter <= bytecounter+1;
		fifo_wr <= '1';
		fifo_in <= std_logic_vector(bytecounter);
	end if;
end process;
   
process begin   
	wait until rising_edge(CLKOUT);
	blinkcounter_60 <= blinkcounter_60 +1;
	TXE_buff <= TXE;
	
	WR <= '1';
	WR_buff <= '1';
	fifo_rd <= '0';
	if TXE = '0' then
		if TXE_buff = '0' then
			if fifo_empty = '1' then
				fifo_rd <= '1';
				if fifo_valid = '1' then --------neu!
					data_0(7 downto 0) <= fifo_out;
					data_0(8) <= fifo_valid;
				end if;
			else
				data_0 <= (others => '0');
			end if;
		end if;

		if TXE_buff = '0' then
			if data_0(8) = '1' then
				data_1 <= data_0;
				D <= data_0(7 downto 0);
				WR <= TXE;
				WR_buff <= TXE;
			end if;
		end if;
	end if;
	
	if TXE = '0' and TXE_buff = '1' then
		if data_1(8) = '1' then
			D <= data_1(7 downto 0);
			WR <= TXE;
			WR_buff <= TXE;
		end if;
		data_1 <= data_0;
	end if;
	
	if TXE = '1' and TXE_buff = '0' then
		if WR_buff = '1' and data_0(8) = '0' then
			data_1 <= (others => '0');
		end if;
	end if;
	
end process;

end behav;